@@ -1,8 +1,7 @@
-from operator import truediv
-import utils.utils as utils
-from utils.CustomTx import CustomTxType
-# import defichainUtils.utils.utils as utils
-# from defichainUtils.utils.CustomTx import CustomTxType
+# import utils.utils as utils
+# from utils.CustomTx import CustomTxType
+import defichainUtils.utils.utils as utils
+from defichainUtils.utils.CustomTx import CustomTxType
 from dataclasses import asdict
 
 
@@ -12,9 +11,13 @@ def decodeCustomTx(hex):
     ''' 
     functionMarker = utils.hexToString(hex[:2])
     if functionMarker == 'j':
-        res = decodeSetGovVariableHeight(hex[2:])
+        res = decodeSetGovVariable(hex[2:],True)
         if res != None:
             res['name'] = 'setgovheight'
+    elif functionMarker == 'G':
+        res = decodeSetGovVariable(hex[2:],False)
+        if res != None:
+            res['name'] = 'setgovvariable'
     elif functionMarker == 's':
         res = decodePoolSwap(hex[2:])
         if res != None:
@@ -82,8 +85,7 @@ def decodeCustomTx(hex):
     elif functionMarker == 'S':
         res = decodeDepositToVault(hex[2:])
         if res != None:
-            res['name'] = 'deposittovault'
-            
+            res['name'] = 'deposittovault'          
     else:
         c = CustomTxType()
         for key, value in asdict(c).items():
@@ -96,40 +98,88 @@ def decodeCustomTx(hex):
     return res
 
 
-def decodeSetGovVariableHeight(hex):
+def decodeSetGovVariable(hex,withHeight):
     # Only data part of hex string!
     govVariable = hex[2:2+2*int(hex[:2],16)]
     hex = hex[2+2*int(hex[:2],16):]
     if utils.stringToHex('ATTRIBUTES') == govVariable:
-        # if hex[:2] != '01':
-        #     return {
-        #         'error': 1,
-        #         'msg': 'Expected only one value in this function. Please contact package DEV.'
-        #     }
-        variables = [
-            {'key':'attributes', 'type': [
-                {'bytesCount': 4, 'key': 'unknown', 'type': int,'endian':None},
-                {'bytesCount': 1, 'key': 'type', 'type': str,'endian':None,'abort':['o']},
-                {'bytesCount': 1, 'key': 'typeId', 'type': int,'endian':None},
-                {'bytesCount': 4, 'key': 'key', 'type': str,'endian':'big'},
-                {'bytesCount': 4, 'key': 'keyId', 'type': int,'endian':'little'},
-                {'bytesCount': 8, 'key': 'value', 'type': int,'endian':'little'}]
-            },
-            {'bytesCount': 4, 'key': 'height', 'type': int,'endian':'little'}
-        ]
-        return searchHex(variables,hex)
+        cnt = int(hex[:2],16)
+        hex = hex[2:]
+        result = {
+            'govvariable': 'ATTRIBUTES',
+            'attributes': [],
+        }
+        for i in range(0,cnt):
+            typeMarker = utils.hexToString(hex[8:10])
+            if typeMarker in ['p']:
+                variables = [
+                    {'bytesCount': 1, 'key': 'type', 'type': str,'endian':None,'abort':['o']},
+                    {'bytesCount': 1, 'key': 'typeId', 'type': int,'endian':None},
+                    {'bytesCount': 4, 'key': 'key', 'type': str,'endian':'big'},
+                    {'bytesCount': 4, 'key': 'keyId', 'type': int,'endian':'little'},
+                    {'bytesCount': 'rest', 'key': 'value', 'type': int,'endian':'little'}
+                ]
+                result['attributes'].append(searchHex(variables,hex[8:44]))
+                hex = hex[44:]
+            elif typeMarker in ['L']:
+                variables = [
+                    {'bytesCount': 1, 'key': 'type', 'type': str,'endian':None,'abort':['o']},
+                    {'bytesCount': 1, 'key': 'key', 'type': str,'endian':None},
+                    {'bytesCount': 4, 'key': 'keyId', 'type': int,'endian':None},
+                    {'bytesCount': 4, 'key': 'unknown1', 'type': int,'endian':'little'},
+                    {'bytesCount': 1, 'key': 'unknown2', 'type': int,'endian':'little'}
+                ]
+                result['attributes'].append(searchHex(variables,hex[8:30]))
+                hex = hex[30:]
+            elif typeMarker in ['o']: 
+                variables = [
+                    {'bytesCount': 1, 'key': 'type', 'type': str,'endian':None},
+                    {'bytesCount': 1, 'key': 'typeId', 'type': str,'endian':None},
+                    {'bytesCount': 7, 'key': 'unkown', 'type': 'hex','endian':None},
+                    {'bytesCount': 4, 'key': 'splitType', 'type': int,'endian':None},
+                    {'bytesCount': 4, 'key': 'tokenId', 'type': int,'endian':'little'},
+                    {'bytesCount': 4, 'key': 'multiplier', 'type': int,'endian':'little'}
+                ]
+                result['attributes'].append(searchHex(variables,hex[8:50]))
+                hex = hex[50:]
+
+        if withHeight: 
+            result['height'] = int(utils.convert_hex(hex[-8:],'little','big'),16)
+            #{'bytesCount': 4, 'key': 'height', 'type': int,'endian':'little'}
+        
+        # variables = [
+        #     {'key':'govvariable', 'bytesCount':0, 'type': 'constant', 'value': 'ATTRIBUTES','endian':None},
+        #     {'key':'attributes', 'type': [
+        #         {'bytesCount': 4, 'key': 'unknown', 'type': int,'endian':None},
+        #         {'bytesCount': 1, 'key': 'type', 'type': str,'endian':None,'abort':['o']},
+        #         {'bytesCount': 1, 'key': 'typeId', 'type': int,'endian':None},
+        #         {'bytesCount': 4, 'key': 'key', 'type': str,'endian':'big'},
+        #         {'bytesCount': 4, 'key': 'keyId', 'type': int,'endian':'little'},
+        #         {'bytesCount': 8, 'key': 'value', 'type': int,'endian':'little'}]
+        #     },
+        #     {'bytesCount': 4, 'key': 'height', 'type': int,'endian':'little'}
+        # ]
+        return result 
     elif utils.stringToHex('ICX_TAKERFEE_PER_BTC') == govVariable:
-        # TODO: write Decoding
-        pass
+        return {
+            'govvariable': 'ICX_TAKERFEE_PER_BTC',
+            'hex': hex
+        }
     elif utils.stringToHex('LP_LOAN_TOKEN_SPLITS') == govVariable:
-        # TODO: write Decoding
-        pass
+        return {
+            'govvariable': 'LP_LOAN_TOKEN_SPLITS',
+            'hex': hex
+        }
     elif utils.stringToHex('LP_SPLITS') == govVariable:
-        # TODO: write Decoding
-        pass
+        return {
+            'govvariable': 'LP_SPLITS',
+            'hex': hex
+        }
     elif utils.stringToHex('ORACLE_DEVIATION') == govVariable:
-        # TODO: write Decoding
-        pass
+        return {
+            'govvariable': 'ORACLE_DEVIATION',
+            'hex': hex
+        }
     else:
         return None
 
@@ -211,7 +261,7 @@ def decodeAccountToAccount(hex):
     return searchHex(variables,hex)
 def decodeTakeLoan(hex):
     variables = [
-        {'bytesCount': 32,'key': 'vault', 'type': 'hex','endian':'little'},
+        {'bytesCount': 32,'key': 'vaultId', 'type': 'hex','endian':'little', 'keepLeadingZeros': True},
         {'key': 'to', 'type': 'address','endian':None},
         {'key': 'amounts', 'type': [
             {'bytesCount': 4,'key': 'tokenId', 'type': int,'endian':'little'},
@@ -221,7 +271,7 @@ def decodeTakeLoan(hex):
     return searchHex(variables,hex)
 def decodePaybackLoan(hex):
     variables = [
-        {'bytesCount': 32,'key': 'vault', 'type': 'hex','endian':'little'},
+        {'bytesCount': 32,'key': 'vaultId', 'type': 'hex','endian':'little', 'keepLeadingZeros': True},
         {'key': 'from', 'type': 'address','endian':None},
         {'key': 'amounts', 'type': [
             {'bytesCount': 4,'key': 'tokenId', 'type': int,'endian':'little'},
@@ -385,6 +435,8 @@ def getValue(variable,hex):
         if variable['bytesCount'] == 'rest':
             hexOfInterest = hex
             hex = ''
+        elif variable['bytesCount'] == 0:
+            hexOfInterest = ''
         else:
             hexOfInterest = hex[:2*variable['bytesCount']]
             hex = hex[2*variable['bytesCount']:]
@@ -420,6 +472,8 @@ def getValue(variable,hex):
         valueOfInterest = utils.decodeHexToAddress(hexOfInterest,addressType,addressVersion)
     elif variable['type'] == 'hex':
         valueOfInterest = hexOfInterest
+    elif variable['type'] == 'constant':
+        valueOfInterest = variable['value']
     else:
         valueOfInterest = None
     
